//=============================================================================
// AnimatedBar.js
//=============================================================================

/*:
 * @plugindesc Hime animated bar
 * @author Hime
 *
 * @param Width
 * @desc Bar width.
 * @type number
 * @default 180
 *
 * @param Hide timer
 * @desc Bar hide timer (seconds).
 * @type number
 * @default 3
 *
 * @param Animation timer
 * @desc Bar animation timer (seconds).
 * @type number
 * @default 1
 *
 * @param Animation delay
 * @desc Bar animation delay (seconds).
 * @type number
 * @default 1
 *
 * @param Max value
 * @desc Max bar value.
 * @type number
 * @default 100
 *
 * @help
 *
 * Plugin Command:
 *   AnimatedBar { "variableId": 1, "delta": -5, "faceIconId": 16, "iconId": 48, "x": 150, "y": 250, "color1": "red", "color2": "blue" } # x and y are optional (0 by default)
 */

;(function() {
    'use strict'

    var parameters = PluginManager.parameters('AnimatedBar')
    var barWidth = Number(parameters['Width']) || 180
    var hideTimer = (Number(parameters['Hide timer']) || 3) * 60
    var animationTimer = (Number(parameters['Animation timer']) || 1) * 60
    var animationDelay = (Number(parameters['Animation delay']) || 1) * 60
    var maxValue = Number(parameters['Max value']) || 100

    //=============================================================================
    // EventQueue
    //=============================================================================
    function EventQueue() {
        throw new Error('This is a static class')
    }

    EventQueue._queue = []

    EventQueue.push = function(event) {
        this._queue.push(event)
    }

    EventQueue.shift = function() {
        return this._queue.shift()
    }

    EventQueue.hasNext = function() {
        return this._queue.length
    }

    //=============================================================================
    // Scene_Map
    //=============================================================================
    var Scene_Map_initialize = Scene_Map.prototype.initialize
    Scene_Map.prototype.initialize = function() {
        Scene_Map_initialize.call(this)
        this._animatedBarOptions = {
            textWidth: 40,
            busy: false,
            hideTimer: null,
            animationTimer: null,
            event: null
        }
    }

    var Scene_Map_createDisplayObjects = Scene_Map.prototype.createDisplayObjects
    Scene_Map.prototype.createDisplayObjects = function() {
        Scene_Map_createDisplayObjects.call(this)
        this.createAnimatedBar()
    }

    Scene_Map.prototype.animatedBarMargin = function() {
        return 6
    }

    Scene_Map.prototype.topMargin = function() {
        return 48
    }

    Scene_Map.prototype.createAnimatedBar = function() {
        this._animatedBarOptions.width = barWidth + 2 * Window_Base._iconWidth + 2 * this.animatedBarMargin()
        this._animatedBarOptions.height = Window_Base._iconHeight + this.topMargin()

        this._animatedBarOptions.animatedBarBackgroundSprite = new Sprite(new Bitmap(this._animatedBarOptions.width, this._animatedBarOptions.height))
        this._animatedBarOptions.animatedBarInnerSprite = new Sprite(new Bitmap(barWidth - 2, Window_Base._iconHeight - 2))
        this._animatedBarOptions.animatedBarDeltaSprite = new Sprite(new Bitmap(48, 32))

        this.hideAnimatedBar()
        this.addChild(this._animatedBarOptions.animatedBarBackgroundSprite)
        this.addChild(this._animatedBarOptions.animatedBarInnerSprite)
        this.addChild(this._animatedBarOptions.animatedBarDeltaSprite)
    }

    var Scene_Map_update = Scene_Map.prototype.update
    Scene_Map.prototype.update = function() {
        Scene_Map_update.call(this)

        if (!this._animatedBarOptions.busy && EventQueue.hasNext()) {
            this._animatedBarOptions.hideTimer = hideTimer

            this._animatedBarOptions.event = EventQueue.shift()
            this._animatedBarOptions.initialValue = $gameVariables.value(this._animatedBarOptions.event.variableId)
            this._animatedBarOptions.variableValue = this._animatedBarOptions.initialValue

            let value = Math.max(0, this._animatedBarOptions.initialValue + this._animatedBarOptions.event.delta)
            $gameVariables.setValue(this._animatedBarOptions.event.variableId, value)

            this._animatedBarOptions.animatedBarBackgroundSprite.move(this._animatedBarOptions.event.x || 0, this._animatedBarOptions.event.y || 0)
            this._animatedBarOptions.animatedBarInnerSprite.move(
                (this._animatedBarOptions.event.x || 0) + Window_Base._iconWidth + this.animatedBarMargin() + 1,
                (this._animatedBarOptions.event.y || 0) + this._animatedBarOptions.height - Window_Base._iconHeight + 1
            )
            this._animatedBarOptions.animatedBarDeltaSprite.move(
                (this._animatedBarOptions.event.x || 0) + (this._animatedBarOptions.width - 48) / 2,
                (this._animatedBarOptions.event.y || 0) + this.topMargin()
            )

            this.drawAnimatedBarFace()
            this.drawAnimatedBarIcon()
            this.drawAnimatedBarBackground()
            this.drawAnimatedBarDelta()

            this.updateAnimatedBarInner()

            this._animatedBarOptions.busy = true
            this.showAnimatedBar()
        }

        if (this._animatedBarOptions.hideTimer !== null && this._animatedBarOptions.hideTimer <= 0) {
            this._animatedBarOptions.busy = false
            this.hideAnimatedBar()
            this.clearAnimatedBar()
            this._animatedBarOptions.hideTimer = null
        } else if (this._animatedBarOptions.hideTimer !== null) {
            if (this._animatedBarOptions.hideTimer === hideTimer - animationDelay) {
                this._animatedBarOptions.animationTimer = animationTimer
                this._animatedBarOptions.piece = this._animatedBarOptions.event.delta / this._animatedBarOptions.animationTimer
            }
            this._animatedBarOptions.hideTimer--
        }

        if (this._animatedBarOptions.event) {
            if (this._animatedBarOptions.animationTimer !== null && this._animatedBarOptions.animationTimer <= 0) {
                this._animatedBarOptions.event = null
                this._animatedBarOptions.animationTimer = null
                delete this._animatedBarOptions.initialValue
                delete this._animatedBarOptions.variableValue
            } else if (this._animatedBarOptions.animationTimer !== null) {
                this._animatedBarOptions.variableValue += this._animatedBarOptions.piece
                this.updateAnimatedBarInner()
                this._animatedBarOptions.animationTimer--
            }
        }
    }

    Scene_Map.prototype.drawAnimatedBarFace = function() {
        this.drawIcon(this._animatedBarOptions.event.faceIconId, 0, this._animatedBarOptions.height - Window_Base._iconHeight)
    }

    Scene_Map.prototype.drawAnimatedBarIcon = function() {
        this.drawIcon(
            this._animatedBarOptions.event.iconId,
            this._animatedBarOptions.width - Window_Base._iconWidth,
            this._animatedBarOptions.height - Window_Base._iconHeight
        )
    }

    Scene_Map.prototype.drawAnimatedBarBackground = function() {
        this._animatedBarOptions.animatedBarBackgroundSprite.bitmap.fillRect(
            Window_Base._iconWidth + this.animatedBarMargin(),
            this._animatedBarOptions.height - Window_Base._iconHeight,
            barWidth,
            Window_Base._iconHeight,
            '#303030'
        )
    }

    Scene_Map.prototype.updateAnimatedBarInner = function() {
        this._animatedBarOptions.animatedBarInnerSprite.bitmap.clear()
        let value = Math.max(0, (this._animatedBarOptions.variableValue * this._animatedBarOptions.animatedBarInnerSprite.bitmap.width) / maxValue)
        this._animatedBarOptions.animatedBarInnerSprite.bitmap.gradientFillRect(
            0,
            0,
            value,
            this._animatedBarOptions.animatedBarInnerSprite.height,
            this._animatedBarOptions.event.color1,
            this._animatedBarOptions.event.color2
        )
    }

    Scene_Map.prototype.drawAnimatedBarDelta = function() {
        this._animatedBarOptions.animatedBarDeltaSprite.bitmap.clear()
        var text = this._animatedBarOptions.event.delta > 0 ? '+' + this._animatedBarOptions.event.delta : this._animatedBarOptions.event.delta
        this._animatedBarOptions.animatedBarDeltaSprite.bitmap.drawText(
            text,
            0,
            0,
            this._animatedBarOptions.animatedBarDeltaSprite.bitmap.width,
            this._animatedBarOptions.animatedBarDeltaSprite.bitmap.height,
            'center'
        )
    }

    Scene_Map.prototype.showAnimatedBar = function() {
        this._animatedBarOptions.animatedBarBackgroundSprite.visible = true
        this._animatedBarOptions.animatedBarInnerSprite.visible = true
        this._animatedBarOptions.animatedBarDeltaSprite.visible = true
    }

    Scene_Map.prototype.hideAnimatedBar = function() {
        this._animatedBarOptions.animatedBarBackgroundSprite.visible = false
        this._animatedBarOptions.animatedBarInnerSprite.visible = false
        this._animatedBarOptions.animatedBarDeltaSprite.visible = false
    }

    Scene_Map.prototype.clearAnimatedBar = function() {
        this._animatedBarOptions.animatedBarBackgroundSprite.bitmap.clear()
        this._animatedBarOptions.animatedBarInnerSprite.bitmap.clear()
        this._animatedBarOptions.animatedBarDeltaSprite.bitmap.clear()
    }

    Scene_Map.prototype.drawIcon = function(iconIndex, x, y) {
        var bitmap = ImageManager.loadSystem('IconSet')
        var pw = Window_Base._iconWidth
        var ph = Window_Base._iconHeight
        var sx = (iconIndex % 16) * pw
        var sy = Math.floor(iconIndex / 16) * ph
        this._animatedBarOptions.animatedBarBackgroundSprite.bitmap.blt(bitmap, sx, sy, pw, ph, x, y)
    }

    //=============================================================================
    // Game Interpreter
    //=============================================================================
    var Game_Interpreter_pluginCommand = Game_Interpreter.prototype.pluginCommand
    Game_Interpreter.prototype.pluginCommand = function(command, args) {
        Game_Interpreter_pluginCommand.call(this, command, args)
        if (command === 'AnimatedBar') {
            var json = args.join(' ')
            try {
                var event = JSON.parse(json)
                EventQueue.push(event)
            } catch (e) {
                console.error('Invalid JSON ' + json)
            }
        }
    }
})()
