//=============================================================================
// Gacha.js
//
// (c)2016 KADOKAWA CORPORATION./YOJI OJIMA
//=============================================================================

/*:
 * @plugindesc Get the item at random
 * @author Takeya Kimura
 *
 * @param Help Message Text
 * @desc The help message for gacha window. "Required Amount" is replaced with the Required Amount.
 * @default 1回Required Amount\Gでガチャを引きます
 *
 * @param Button Text
 * @desc The button text for gacha commands.
 * @default ガチャを引く
 *
 * @param Get Message Text
 * @desc The message of After receiving. "Item Name" is replaced with the received item name.
 * @default GET Item Name
 *
 * @param Show Item Description
 * @desc The switch of item description display
 * @default 0
 *
 * @param Effect
 * @desc The animation number for get effect.
 * @default 119,120,121
 * @require 1
 *
 * @param Rank1 Effect
 * @desc The animation number for rank 1 effect. If you specify -1, does not display the animation.
 * @default -1
 * @require 1
 * @type animation
 *
 * @param Rank2 Effect
 * @desc The animation number for rank 2 effect. If you specify -1, does not display the animation.
 * @default -1
 * @require 1
 * @type animation
 *
 * @param Rank3 Effect
 * @desc The animation number for rank 3 effect. If you specify -1, does not display the animation.
 * @default -1
 * @require 1
 * @type animation
 *
 * @param Rank4 Effect
 * @desc The animation number for rank 4 effect. If you specify -1, does not display the animation.
 * @default -1
 * @require 1
 * @type animation
 *
 * @param Rank5 Effect
 * @desc The animation number for rank 5 effect. If you specify -1, does not display the animation.
 * @default -1
 * @require 1
 * @type animation
 *
 * @param ME
 * @desc The ME name for get music effect.
 * @default Organ
 * @require 1
 * @dir audio/me/
 * @type file
 *
 * @param  Required Amount
 * @desc The Gold for gacha.
 * @default 100
 *
 * @noteParam gachaImage
 * @noteRequire 1
 * @noteDir img/gacha/
 * @noteType file
 * @noteData items
 *
 * @help
 *
 * Plugin Command:
 *   Gacha open                 # Open the Gacha screen
 *   Gacha add item 1           # Add item #1 to the Gacha
 *   Gacha remove item 1        # Remove item #1 from the Gacha
 *   Gacha clear                # Clear the Gacha
 *
 *
 * Item Note:
 *   <gachaImage:image>        # Gacha image file name. Please image put in "img/gacha/" folder.
 *   <gachaNumLot:10>          # The number of the lottery.
 *   <gachaRank:5>             # The rank of the item(1-5).
 */

(function () {

    var parameters = PluginManager.parameters('Gacha');

    var BUTTON_TEXT = String(parameters['Button Text'] || 'ガチャを引く');
    var GET_TEXT = String(parameters['Get Message Text'] || 'GET Item Name');
    var itemDescEnable = !!Number(parameters['Show Item Description'] || 0);
    var EFFECT = String(parameters['Effect'] || '119,120,121').split(/\s*,\s*/);
    var RANK_EFFECT = [
        Number(parameters['Rank1 Effect'] || '-1'),
        Number(parameters['Rank2 Effect'] || '-1'),
        Number(parameters['Rank3 Effect'] || '-1'),
        Number(parameters['Rank4 Effect'] || '-1'),
        Number(parameters['Rank5 Effect'] || '-1')
    ];
    var AUDIO_EFFECT = String(parameters['ME'] || 'Organ');
    var GACHA_PRICE = Number(parameters['Required Amount'] || '100');

    //=============================================================================
    // Game_Interpreter
    //=============================================================================

    var _Game_Interpreter_pluginCommand = Game_Interpreter.prototype.pluginCommand;
    Game_Interpreter.prototype.pluginCommand = function (command, args) {
        _Game_Interpreter_pluginCommand.call(this, command, args);
        if (command === "Gacha") {
            switch (args[0]) {
                case "open":
                    SceneManager.push(Scene_Gacha);
                    break;

                case 'add':
                    $gameSystem.addToGacha(args[1], Number(args[2]));
                    break;

                case 'remove':
                    $gameSystem.removeFromGacha(args[1], Number(args[2]));
                    break;

                case 'clear':
                    $gameSystem.clearGacha();
                    break;
            }
        }
    };

    //=============================================================================
    // Game_System
    //=============================================================================

    Game_System.prototype.addToGacha = function (type, dataId) {
        if (!this._GachaFlags) {
            this.clearGacha();
        }
        var typeIndex = this.gachaTypeToIndex(type);
        if (typeIndex >= 0) {
            this._GachaFlags[typeIndex][dataId] = true;
        }
    };

    Game_System.prototype.removeFromGacha = function (type, dataId) {
        if (this._GachaFlags) {
            var typeIndex = this.gachaTypeToIndex(type);
            if (typeIndex >= 0) {
                this._GachaFlags[typeIndex][dataId] = false;
            }
        }
    };

    Game_System.prototype.gachaTypeToIndex = function (type) {
        switch (type) {
            case 'item':
                return 0;
            case 'weapon':
                return 1;
            case 'armor':
                return 2;
            default:
                return -1;
        }
    };

    Game_System.prototype.clearGacha = function () {
        this._GachaFlags = [[], [], []];
    };

    Game_System.prototype.isInGacha = function (item) {
        if (this._GachaFlags && item) {
            var typeIndex = -1;
            if (DataManager.isItem(item)) {
                typeIndex = 0;
            } else if (DataManager.isWeapon(item)) {
                typeIndex = 1;
            } else if (DataManager.isArmor(item)) {
                typeIndex = 2;
            }
            if (typeIndex >= 0) {
                return !!this._GachaFlags[typeIndex][item.id];
            } else {
                return false;
            }
        } else {
            return false;
        }
    };

    //=============================================================================
    // Window_GachaTurn
    //=============================================================================

    function Window_GachaTurn() {
        this.initialize.apply(this, arguments);
    }

    Window_GachaTurn.prototype = Object.create(Window_Command.prototype);
    Window_GachaTurn.prototype.constructor = Window_GachaTurn;

    Window_GachaTurn.prototype.initialize = function (lotIsValid) {
        this._lotIsValid = lotIsValid;
        Window_Command.prototype.initialize.call(this, 0, 0);
        this.updatePlacement();
    };

    Window_GachaTurn.prototype.windowWidth = function () {
        return 164;
    };

    Window_GachaTurn.prototype.windowHeight = function () {
        return this.fittingHeight(2);
    };

    Window_GachaTurn.prototype.updatePlacement = function () {
        this.x = Graphics.boxWidth - 5 * 24 - this.width / 2;
        this.y = 48;
    };

    Window_GachaTurn.prototype.maxCols = function () {
        return 1;
    };

    Window_GachaTurn.prototype.makeCommandList = function () {
        this.addCommand(BUTTON_TEXT + ' (' + GACHA_PRICE + ' ' + TextManager.currencyUnit + ')', 'gacha', $gameParty.gold() >= GACHA_PRICE && !!this._lotIsValid);
        this.addCommand(TextManager.cancel, 'cancel');
    };

    Window_GachaTurn.prototype.itemTextAlign = function () {
        return 'center';
    };

    Window_GachaTurn.prototype.standardPadding = function () {
        return 8;
    };

    //=============================================================================
    // Window_GachaClose
    //=============================================================================

    function Window_GachaClose() {
        this.initialize.apply(this, arguments);
    }

    Window_GachaClose.prototype = Object.create(Window_Command.prototype);
    Window_GachaClose.prototype.constructor = Window_GachaClose;

    Window_GachaClose.prototype.initialize = function (lotIsValid) {
        this._lotIsValid = lotIsValid;
        Window_Command.prototype.initialize.call(this, 0, 0);
    };

    Window_GachaClose.prototype.windowWidth = function () {
        return Graphics.boxWidth;
    };

    Window_GachaClose.prototype.windowHeight = function () {
        return this.fittingHeight(1);
    };

    Window_GachaClose.prototype.maxCols = function () {
        return 2;
    };

    Window_GachaClose.prototype.makeCommandList = function () {
        this.addCommand(BUTTON_TEXT + ' (' + GACHA_PRICE + ' ' + TextManager.currencyUnit + ')', 'gacha', $gameParty.gold() >= GACHA_PRICE && !!this._lotIsValid);
        this.addCommand(TextManager.cancel, 'cancel');
    };

    //=============================================================================
    // Window_GachaCard
    //=============================================================================

    function Window_GachaCard() {
        this.initialize.apply(this, arguments);
    }

    Window_GachaCard.prototype = Object.create(Window_Base.prototype);
    Window_GachaCard.prototype.constructor = Window_GachaCard;

    Window_GachaCard.prototype.initialize = function (height) {
        this._height = height;
        Window_Base.prototype.initialize.call(this, 0, 0, this.windowWidth(), this.windowHeight());

        this._gachaSprite = new Sprite();
        this._gachaSprite.anchor.x = 0;
        this._gachaSprite.anchor.y = 0;
        this.addChildToBack(this._gachaSprite);
        this.windowskin = new Bitmap();

        this._rankSprite = new Sprite_GachaEffect();
        this._rankSprite.keepDisplay(true);
        this.addChildToBack(this._rankSprite);
        this.windowskin = new Bitmap();
    };

    Window_GachaCard.prototype.windowWidth = function () {
        return Graphics.boxWidth;
    };

    Window_GachaCard.prototype.windowHeight = function () {
        return this._height;
    };

    Window_GachaCard.prototype.setItem = function (item) {
        if (this._item !== item) {
            this._rankSprite.allRemove();
            this._item = item;
            this.refresh();
        }
    };

    Window_GachaCard.prototype.refresh = function () {
        if (!this._item || !this._item.meta.gachaImage) {
            this._gachaSprite.bitmap = null;
        } else {
            var bitmap = ImageManager.loadBitmap("img/gacha/", this._item.meta.gachaImage);
            this._gachaSprite.bitmap = bitmap;
            bitmap.smooth = true;
        }

        if (!!this._item.meta.gachaImage) {
            var rank = Number(this._item.meta.gachaRank || '-1');
            if (rank > 0 && rank < 6) {
                if (RANK_EFFECT[rank - 1] > 0) {
                    var animation = $dataAnimations[RANK_EFFECT[rank - 1]];
                    this._rankSprite.startAnimation(animation, false, 0);
                }
            }
        }
    };

    Window_GachaCard.prototype.update = function () {
        Window_Base.prototype.update.call(this);
        if (this._gachaSprite.bitmap) {
            this._gachaSprite.x = (this.windowWidth() - this._gachaSprite.bitmap.width) / 2;
            this._gachaSprite.y = (this.windowHeight() - this._gachaSprite.bitmap.height) / 2;

            this._rankSprite.x = this.windowWidth() / 2;
            this._rankSprite.y = (this.windowHeight() - this._gachaSprite.bitmap.height) / 2 + this._gachaSprite.bitmap.height + 75;
        }
    };

    Window_GachaCard.prototype.standardPadding = function () {
        return 0;
    };

    //=============================================================================
    // Window_HelpCentered
    //=============================================================================

    function Window_HelpCentered() {
        this.initialize.apply(this, arguments);
    }

    Window_HelpCentered.prototype = Object.create(Window_Help.prototype);
    Window_HelpCentered.prototype.constructor = Window_HelpCentered;

    Window_HelpCentered.prototype.initialize = function (lines) {
        Window_Help.prototype.initialize.call(this, lines);
    };

    Window_HelpCentered.prototype.refresh = function () {
        this.contents.clear();
        this.drawText(this._text, 0, 0, Graphics.boxWidth, 'center');
    };

    //=============================================================================
    // Scene_Gacha
    //=============================================================================

    function Scene_Gacha() {
        this.initialize.apply(this, arguments);
    }

    Scene_Gacha.prototype = Object.create(Scene_MenuBase.prototype);
    Scene_Gacha.prototype.constructor = Scene_Gacha;

    Scene_Gacha.prototype.initialize = function () {
        Scene_MenuBase.prototype.initialize.call(this);

        this._lot = [];
        var numLot;
        var item, i, j;
        for (i = 1; i < $dataItems.length; i++) {
            item = $dataItems[i];
            if ($gameSystem.isInGacha(item)) {
                numLot = Number(item.meta.gachaNumLot || '0');
                for (j = 0; j < numLot; j++) {
                    this._lot.push(item);
                }
            }
        }
        for (i = 1; i < $dataWeapons.length; i++) {
            item = $dataWeapons[i];
            if ($gameSystem.isInGacha(item)) {
                numLot = Number(item.meta.gachaNumLot || '0');
                for (j = 0; j < numLot; j++) {
                    this._lot.push(item);
                }
            }
        }
        for (i = 1; i < $dataArmors.length; i++) {
            item = $dataArmors[i];
            if ($gameSystem.isInGacha(item)) {
                numLot = Number(item.meta.gachaNumLot || '0');
                for (j = 0; j < numLot; j++) {
                    this._lot.push(item);
                }
            }
        }
    };

    Scene_Gacha.prototype.create = function () {
        Scene_MenuBase.prototype.create.call(this);

        this.createTurnWindow();
        this.createDescriptionWindow();
        this.createMessageWindow();
        this.createCardWindow();

        this._windowFadeSprite = new ScreenSprite();
        this._windowFadeSprite.setColor(0, 0, 0);
        this._windowFadeSprite.opacity = 0;
        this._windowFadeSprite.visible = false;
        this.addChild(this._windowFadeSprite);

        this._effectSprite = new Sprite_GachaEffect();
        this._effectSprite.x = Graphics.boxWidth / 2;
        this._effectSprite.y = Graphics.boxHeight / 2;
        this.addChild(this._effectSprite);
    };

    Scene_Gacha.prototype.createTurnWindow = function () {
        this._turnWindow = new Window_GachaTurn(this._lot.length > 0);
        this._turnWindow.setHandler('gacha', this.commandGacha.bind(this));
        this._turnWindow.setHandler('cancel', this.popScene.bind(this));
        this.addWindow(this._turnWindow);
    };

    Scene_Gacha.prototype.createDescriptionWindow = function () {
        this._descriptionWindow = new Window_HelpCentered(1);
        this.addWindow(this._descriptionWindow);
        this._descriptionWindow.hide();
    };

    Scene_Gacha.prototype.createMessageWindow = function () {
        this._messageWindow = new Window_HelpCentered(1);
        this._messageWindow.y = Graphics.boxHeight - this._messageWindow.height;
        this.addWindow(this._messageWindow);
        this._messageWindow.hide();
    };

    Scene_Gacha.prototype.createCardWindow = function () {
        var height = Graphics.boxHeight - this._turnWindow.height - this._descriptionWindow.height - this._messageWindow.height;
        this._cardWindow = new Window_GachaCard(height);
        this._cardWindow.y = this._descriptionWindow.height;
        this.addWindow(this._cardWindow);
        this._cardWindow.hide();
    };

    Scene_Gacha.prototype.commandGacha = function () {
        this._turnWindow.deactivate();

        // Draw lots
        if (this._lot.length <= 0) {
            this._item = null;
        } else {
            this._item = this._lot[(Math.random() * this._lot.length) >> 0];
            $gameParty.loseGold(GACHA_PRICE);
            $gameParty.gainItem(this._item, 1);
        }

        if (this._item) {

            this._screenFadeOut();
            this._effectPlaying = true;
            this._wait = 0;

            var animation = $dataAnimations[EFFECT[Math.floor(Math.random() * EFFECT.length)]];
            this._effectSprite.startAnimation(animation, false, 0);

            AudioManager.playMe({ "name": AUDIO_EFFECT, "volume": 90, "pitch": 100, "pan": 0 });
        }
    };

    Scene_Gacha.prototype.update = function () {
        Scene_Menu.prototype.update.call(this);
        this._updateScreenFlashSprite();

        this._wait++;
        if (this._wait > 12) {
            if (this._effectPlaying) {
                if (!this._effectSprite.isAnimationPlaying()) {
                    this._screenFadeIn();

                    if (this._item) {
                        this._cardWindow.setItem(this._item);
                        this._descriptionWindow.setText(this._item.description);
                    }

                    this._effectPlaying = false;

                    this.showResult();
                    this._turnWindow.activate();
                    this._turnWindow.refresh();

                    var message = GET_TEXT;
                    message = message.replace(/Item Name/gi, String(this._item.name));
                    this._messageWindow.setText(message);
                    this._wait = 0;
                    this._resultShowing = true;
                } else if (TouchInput.isTriggered() || Input.isTriggered("ok")) {
                    this._effectSprite.allRemove();
                }
            } else if (this._resultShowing) {
                this._resultShowing = false;
            }
        }
    };

    Scene_Gacha.prototype.showResult = function () {
        this._turnWindow.x = Graphics.boxWidth - this._turnWindow.width;
        this._turnWindow.y = Graphics.boxHeight - this._messageWindow.height - this._turnWindow.height;
        this._descriptionWindow.show();
        this._messageWindow.show();
        this._cardWindow.show();
    };

    Scene_Gacha.prototype._updateScreenFlashSprite = function () {
        var d;
        if (this._screenFadeOutDuration > 0) {
            d = this._screenFadeOutDuration--;
            this._windowFadeSprite.opacity += 255 * (1 - (d - 1) / d);
        }
        if (this._screenFadeInDuration > 0) {
            d = this._screenFadeInDuration--;
            this._windowFadeSprite.opacity *= (d - 1) / d;
            this._windowFadeSprite.visible = (this._screenFadeInDuration > 0);
        }
    };

    Scene_Gacha.prototype._screenFadeOut = function () {
        this._windowFadeSprite.visible = true;
        this._screenFadeOutDuration = 15;
        this._screenFadeInDuration = 0;
    };

    Scene_Gacha.prototype._screenFadeIn = function () {
        this._windowFadeSprite.visible = true;
        this._screenFadeOutDuration = 0;
        this._screenFadeInDuration = 15;
    };

    //=============================================================================
    // Sprite_GachaEffect
    //=============================================================================

    function Sprite_GachaEffect() {
        this.initialize.apply(this, arguments);
    }

    Sprite_GachaEffect.prototype = Object.create(Sprite.prototype);
    Sprite_GachaEffect.prototype.constructor = Sprite_GachaEffect;

    Sprite_GachaEffect.prototype.initialize = function () {
        Sprite.prototype.initialize.call(this);
        this._animationSprites = [];
        this._endSprites = [];
        this._effectTarget = this;
        this._hiding = false;
        this._keepDisplay = false;
    };

    Sprite_GachaEffect.prototype.keepDisplay = function (value) {
        this._keepDisplay = value;
    };

    Sprite_GachaEffect.prototype.update = function () {
        Sprite.prototype.update.call(this);
        this.updateVisibility();
        this.updateAnimationSprites();
    };

    Sprite_GachaEffect.prototype.hide = function () {
        this._hiding = true;
        this.updateVisibility();
    };

    Sprite_GachaEffect.prototype.show = function () {
        this._hiding = false;
        this.updateVisibility();
    };

    Sprite_GachaEffect.prototype.updateVisibility = function () {
        this.visible = !this._hiding;
    };

    Sprite_GachaEffect.prototype.updateAnimationSprites = function () {
        if (this._animationSprites.length > 0) {
            var sprites = this._animationSprites.clone();
            this._animationSprites = [];
            for (var i = 0; i < sprites.length; i++) {
                var sprite = sprites[i];
                if (sprite.isPlaying()) {
                    this._animationSprites.push(sprite);
                } else {
                    if (!this._keepDisplay) {
                        sprite.remove();
                    }
                    else {
                        this._endSprites.push(sprite);
                    }
                }
            }
        }
    };

    Sprite_GachaEffect.prototype.startAnimation = function (animation, mirror, delay) {
        var sprite = new Sprite_Animation();
        sprite.setup(this._effectTarget, animation, mirror, delay);
        this.parent.addChild(sprite);
        this._animationSprites.push(sprite);
    };

    Sprite_GachaEffect.prototype.isAnimationPlaying = function () {
        return this._animationSprites.length > 0;
    };

    Sprite_GachaEffect.prototype.allRemove = function () {
        var sprites, sprite, i;
        if (this._animationSprites.length > 0) {
            sprites = this._animationSprites.clone();
            this._animationSprites = [];
            for (i = 0; i < sprites.length; i++) {
                sprite = sprites[i];
                sprite.remove();
            }
        }
        if (this._endSprites.length > 0) {
            sprites = this._endSprites.clone();
            this._endSprites = [];
            for (i = 0; i < sprites.length; i++) {
                sprite = sprites[i];
                sprite.remove();
            }
        }
    };

})();