/*jslint vars: true, nomen: true, plusplus: true, eqeq: true */
/*globals console*/
/*globals Game_Interpreter, DataManager, PluginManager, ImageManager, AudioManager, Graphics */
/*globals $dataMap, $dataMapInfos, $dataAnimations */
//=============================================================================
// PreloadManager.js
//=============================================================================

/*:
 * @plugindesc Pivert PreloadManager.
 * @author Pivert
 *
 * @param Default preload map resources
 * @desc If you want to preload all the resources on a map by default, set this to true.
 * @default false
 *
 * @param Print Debug to Console
 * @desc If you want to see debug information in the console (F8) set this to true.
 * @default false
 */

(function () {

    'use strict';

    var parameters = PluginManager.parameters('PreloadManager');
    var debug = Boolean(parameters['Print Debug to Console'] === 'true' || false);
    var defaultPreloadMapResources = Boolean(parameters['Default preload map resources'] === 'true' || false);

    if (debug) {
        console.log("========= PreloadManager: Debug mode on =========");
    }

    var PreloadManager = function () {
        throw new Error('This is a static class');
    };

    PreloadManager._ready = false;
    PreloadManager._loadingType = false;
    PreloadManager._callOnComplete = false;
    PreloadManager._filesLoaded = 0;
    PreloadManager._filesTotal = 0;
    PreloadManager._preloadedResources = [];

    var _Game_Interpreter_pluginCommand = Game_Interpreter.prototype.pluginCommand;
    Game_Interpreter.prototype.pluginCommand = function (command, args) {
        _Game_Interpreter_pluginCommand.call(this, command, args);
        if (command === 'PreloadManager') {
            var mapId, eventId, pageId;
            switch (args[0]) {
            case 'preloadMapResources':
                mapId = args[1];
                PreloadManager.setLoadingType('map');
                PreloadManager.preloadMapResources(mapId, function () {
                    PreloadManager.start();
                });
                break;
            case 'preloadEventResources':
                mapId = args[1];
                eventId = args[2];
                PreloadManager.setLoadingType('event');
                PreloadManager.preloadEventResources(mapId, eventId, function () {
                    PreloadManager.start();
                });
                break;
            case 'preloadPageResources':
                mapId = args[1];
                eventId = args[2];
                // -1 because pages are 0 based
                pageId = args[3] - 1;
                PreloadManager.setLoadingType('page');
                PreloadManager.preloadPageResources(mapId, eventId, pageId, function () {
                    PreloadManager.start();
                });
                break;
            }
        }
    };

    PreloadManager.callOnComplete = function (func) {
        this._callOnComplete = func;
    };

    PreloadManager.setLoadingType = function (type) {
        this._loadingType = type;
    };

    PreloadManager.controlIfReady = function (manual) {
        if (this._filesLoaded >= this._filesTotal && (this._ready || this._filesTotal == 0)) {
            if (debug) {
                console.log("All files loaded (" + this._filesLoaded + "/" + this._filesTotal + ")");
            }
            this._filesTotal = this._filesLoaded = 0;
            if (this._callOnComplete) {
                this._callOnComplete.call();
            }
        }
    };

    PreloadManager._increaseFileNums = function () {
        this._ready = false;
        this._filesTotal += 1;
    };

    PreloadManager.onFileLoaded = function (filename) {
        if (debug) {
            console.log("Loaded file: " + filename + "(" + this._filesLoaded + "/" + this._filesTotal + ")");
        }
        this._filesLoaded += 1;
        this.controlIfReady();
    };

    PreloadManager.start = function () {
        this._ready = true;
        this.controlIfReady(true);
    };

    PreloadManager.preloadImages = function (type, filename, hue) {
        if (filename.constructor === Array) {
            var i;
            for (i = 0; i < filename.length; i++) {
                this.preloadImages(type, filename[i], hue);
            }
        } else {
            if (filename.length > 0) {
                if (debug) {
                    console.log("Preloading image (" + type + "): ", filename);
                }
                var func = "load" + type.charAt(0).toUpperCase() + type.substr(1).toLowerCase();
                if (typeof ImageManager[func] === 'undefined') {
                    var e = "PreloadManager: " + type + " is not a valid image load key. Check your configuration.";
                    Graphics.printError(e);
                    throw new Error(e);
                }
                this._increaseFileNums();
                ImageManager[func](filename, hue).addLoadListener(this.onFileLoaded.bind(this, filename));
            }
        }
    };

    PreloadManager.preloadBGM = function (audioObject) {
        this.preloadAudio("bgm", audioObject);
    };

    PreloadManager.preloadBGS = function (audioObject) {
        this.preloadAudio("bgs", audioObject);
    };

    PreloadManager.preloadSE = function (audioObject) {
        this.preloadAudio("se", audioObject);
    };

    PreloadManager.preloadME = function (audioObject) {
        this.preloadAudio("me", audioObject);
    };

    PreloadManager.preloadAudio = function (type, audioObject) {
        if (!audioObject) {
            return;
        }
        if (audioObject.constructor === Array) {
            var i, max;
            for (i = 0, max = audioObject.length; i < max; i++) {
                this.preloadAudio(type, audioObject[i]);
            }
        } else {
            if (audioObject.name.length <= 0) {
                return;
            }
            this._increaseFileNums();
            AudioManager.createBuffer(type, audioObject.name).addLoadListener(this.onFileLoaded.bind(this, audioObject.name));
        }
    };

    PreloadManager.preloadAnimation = function (animationId) {
        var animation = $dataAnimations[animationId];
        this.preloadImages('animation', [animation.animation1Name, animation.animation2Name]);
        var i, max;
        for (i = 0, max = animation.timings.length; i < max; i++) {
            var se = animation.timings[i].se;
            this.preloadSE(se);
        }
    };

    PreloadManager.preloadResource = function (identifier, preloadFunction, anywayFunction) {
        if (this._preloadedResources.indexOf(identifier) > -1) {
            if (debug) {
                console.log("Resource " + identifier + " already preloaded, skipping.");
            }
        } else {
            if (debug) {
                console.log("Resource " + identifier + " preload starting.");
            }
            this._preloadedResources.push(identifier);
            preloadFunction.call(this);
        }
        if (anywayFunction) {
            anywayFunction.call(this);
        }
    };

    PreloadManager.startPreload = function (mapId, event, page, index, listEntry, callback) {

        // Wait and ensure map is loaded
        if (!DataManager.isMapDataLoaded()) {
            return setTimeout(this.preloadEventListEntry.bind(this, mapId, listEntry, callback, 100));
        }

        PreloadManager.preloadResource('audio:bgm:' + $dataMap.bgm.name, function () {
            this.preloadBGM($dataMap.bgm);
        }, PreloadManager.preloadResource('audio:bgs:' + $dataMap.bgs.name, function () {
            this.preloadBGS($dataMap.bgs);
        }, PreloadManager.preloadResource('image:battleback1:' + $dataMap.battleback1Name, function () {
            this.preloadImages('battleback1', $dataMap.battleback1Name);
        }, PreloadManager.preloadResource('image:battleback2:' + $dataMap.bgm.name, function () {
            this.preloadImages('battleback2', $dataMap.battleback2Name);
        }, PreloadManager.preloadResource('image:parallax:' + $dataMap.bgm.name, function () {
            this.preloadImages('parallax', $dataMap.parallaxName);
        }, PreloadManager.preloadResource('image:character:' + page.image.characterName, function () {
            this.preloadImages('character', page.image.characterName);
        }, this.preloadEventListEntry(index, listEntry, callback)))))));
    };

    PreloadManager.preloadEventListEntry = function (index, listEntry, callback) {
        var code = listEntry.code;
        var parameters = listEntry.parameters;
        var self = this;
        switch (code) {
        case 231:
            // Show Picture
            PreloadManager.preloadResource('image:picture:' + parameters[1], function () {
                this.preloadImages('picture', parameters[1]);
            }, function () {
                callback(index);
            });
            break;
        case 241:
            // Play BGM
            PreloadManager.preloadResource('audio:bgm:' + parameters[0].name, function () {
                this.preloadBGM(parameters[0]);
            }, function () {
                callback(index);
            });
            break;
        case 245:
            // Play BGS
            PreloadManager.preloadResource('audio:bgs:' + parameters[0].name, function () {
                this.preloadBGS(parameters[0]);
            }, function () {
                callback(index);
            });
            break;
        case 249:
            // Play ME
            PreloadManager.preloadResource('audio:me:' + parameters[0].name, function () {
                this.preloadME(parameters[0]);
            }, function () {
                callback(index);
            });
            break;
        case 250:
            // Play SE
            PreloadManager.preloadResource('audio:se:' + parameters[0].name, function () {
                this.preloadSE(parameters[0]);
            }, function () {
                callback(index);
            });
            break;
        case 284:
            // Change Parallax
            PreloadManager.preloadResource('image:parallax:' + parameters[0], function () {
                this.preloadImages('parallax', parameters[0]);
            }, function () {
                callback(index);
            });
            break;
        case 101:
            // Show Text
            PreloadManager.preloadResource('image:face:' + parameters[0], function () {
                this.preloadImages('face', parameters[0]);
            }, function () {
                callback(index);
            });
            break;
        case 212:
            // Show Animation
            PreloadManager.preloadResource('image:animation:' + parameters[1], function () {
                this.preloadAnimation(parameters[1]);
            }, function () {
                callback(index);
            });
            break;
        }
    };

    PreloadManager.preloadEventListEntries = function (entries, callback) {
        var cb = function (index) {
            if (index === entries.length - 1) {
                callback();
            }
        };
        var i, max;
        for (i = 0, max = entries.length; i < max; i++) {
            var entry = entries[i];
            if (!entry) {
                continue;
            }
            PreloadManager.preloadEventListEntry(i, entry, cb);
        }
    };

    PreloadManager.preloadMapResources = function (mapId, callback) {
        PreloadManager.preloadResource('map:' + mapId, function () {
            var entries = [];

            var e, eMax;
            for (e = 0, eMax = $dataMap.events.length; e < eMax; e++) {
                var event = $dataMap.events[e];
                if (!event) {
                    continue;
                }
                var p, pMax;
                for (p = 0, pMax = event.pages.length; p < pMax; p++) {
                    var page = event.pages[p];
                    if (!page) {
                        continue;
                    }
                    var l, lMax;
                    for (l = 0, lMax = page.list.length; l < lMax; l++) {
                        var listEntry = page.list[l];
                        if (!listEntry) {
                            continue;
                        }
                        entries.push(listEntry);
                    }
                }
            }

            if (debug) {
                console.log('Starting event list entries preload');
            }
            PreloadManager.preloadEventListEntries(entries, callback);
        }, callback);
    };

    PreloadManager.preloadEventResources = function (mapId, eventId, callback) {
        PreloadManager.preloadResource('event:' + mapId + ':' + eventId, function () {
            var entries = [];

            var event = $dataMap.events[eventId];
            if (!event) {
                return;
            }
            var p, pMax;
            for (p = 0, pMax = event.pages.length; p < pMax; p++) {
                var page = event.pages[p];
                if (!page) {
                    continue;
                }
                var l, lMax;
                for (l = 0, lMax = page.list.length; l < lMax; l++) {
                    var listEntry = page.list[l];
                    if (!listEntry) {
                        continue;
                    }
                    entries.push(listEntry);
                }
            }

            if (debug) {
                console.log('Starting event list entries preload');
            }
            PreloadManager.preloadEventListEntries(entries, callback);
        }, callback);
    };

    PreloadManager.preloadPageResources = function (mapId, eventId, pageId, callback) {
        PreloadManager.preloadResource('page:' + mapId + ':' + eventId + ':' + pageId, function () {
            var entries = [];

            var event = $dataMap.events[eventId];
            if (!event) {
                return;
            }
            var page = event.pages[pageId];
            if (!page) {
                return;
            }
            var l, lMax;
            for (l = 0, lMax = page.list.length; l < lMax; l++) {
                var listEntry = page.list[l];
                if (!listEntry) {
                    continue;
                }
                entries.push(listEntry);
            }

            if (debug) {
                console.log('Starting event list entries preload');
            }
            PreloadManager.preloadEventListEntries(entries, callback);
        }, callback);
    };

    var DataManager_loadMapData = DataManager.loadMapData;
    DataManager.loadMapData = function (mapId) {
        this._mapId = mapId;
        DataManager_loadMapData.call(this, mapId);
    };

    var DataManager_onLoad = DataManager.onLoad;
    DataManager.onLoad = function (object) {
        DataManager_onLoad.call(this, object);
        if ($dataMap) {
            this._preloaded = false;
            PreloadManager.callOnComplete(this._onPreloadDone.bind(this));
            var note = $dataMap.note;
            var preloadMapResources;
            try {
                note = JSON.parse(note);
                preloadMapResources = note.preloadMapResources;
            } catch (e) {
                if (debug) {
                    console.log('Could not parse $dataMap.note for Map #' + this._mapId);
                }
            }
            if ((preloadMapResources === undefined && defaultPreloadMapResources) || preloadMapResources === true) {
                PreloadManager.setLoadingType('map');
                PreloadManager.preloadMapResources(this._mapId, function () {
                    PreloadManager.start();
                });
            }
        }
    };

    var DataManager_isMapLoaded = DataManager.isMapLoaded;
    DataManager.isMapLoaded = function () {
        return (this.isMapDataLoaded() && this._preloaded);
    };

    DataManager._onPreloadDone = function () {
        this._preloaded = true;
    };

    DataManager.isMapDataLoaded = function () {
        return DataManager_isMapLoaded.call(this);
    };

}());
