//=============================================================================
// Puzzle.js
//=============================================================================

/*:
 * @plugindesc Pivert Jigsaw Puzzle.
 * @author Pivert
 *
 * @param Retry text
 * @desc Retry text
 * @type string
 * @default Retry
 *
 * @param Quit text
 * @desc Quit text
 * @type string
 * @default Quit
 *
 * @param Images directory
 * @desc Images directory
 * @type string
 * @default img/pictures
 *
 * @param Images
 * @desc Name of the available images separated by a comma
 * @type string
 * @default
 *
 * @param Tolerance Variable ID
 * @desc Variable ID of the tolerance of the pieces magnetism (percentage of the size of the piece)
 * @type number
 * @default 25
 *
 * @param Victory Sound
 * @desc Victory Sound
 * @type number
 * @default Victory1
 *
 * @param Victory variable ID
 * @desc Victory variable ID
 * @type number
 * @default 1
 *
 * @param Width
 * @desc Width
 * @type number
 * @default 1152
 *
 * @param Height
 * @desc Height
 * @type number
 * @default 864
 *
 * @help
 *
 * Plugin Command:
 *   Puzzle open            # Open the puzzle screen
 *
 * Victory variable
 *   The result of the puzzle is stored this way
 *      <size of the puzzle><tolerance on 3 digits>
 *   e.g.: 12020 if the player chose 12 pieces and the tolerance of 20%
 *
 */

;(function() {
    'use strict'

    var parameters = PluginManager.parameters('Puzzle')
    var retryText = parameters['Retry text'] || 'Retry'
    var quitText = parameters['Quit text'] || 'Quit'
    var imgDirectory = parameters['Images directory'] || 'img/pictures'
    var puzzleImages = (parameters['Images'] || '').split(/,\s*/).map(function(image) {
        return { name: image }
    })
    var toleranceVariableId = Number(parameters['Tolerance Variable ID']) || 25
    var victoryVariableId = Number(parameters['Victory variable ID']) || 1
    var victoryME = parameters['Victory ME'] || 'Victory1'
    var imageWidth = Number(parameters['Width']) || 1152
    var imageHeight = Number(parameters['Height']) || 864

    //=============================================================================
    // Scene_Puzzle
    //=============================================================================
    function Scene_Puzzle() {
        this.initialize.apply(this, arguments)
    }

    Scene_Puzzle.prototype = Object.create(Scene_MenuBase.prototype)
    Scene_Puzzle.prototype.constructor = Scene_Puzzle

    Scene_Puzzle.prototype.initialize = function() {
        Scene_MenuBase.prototype.initialize.call(this)
    }

    Scene_Puzzle.prototype.create = function() {
        Scene_MenuBase.prototype.create.call(this)
        this.createChoseImageCommandWindow()
        this.createChoseSizeCommandWindow()
        this.createPuzzleWindow()
        this.createCommandWindow()
    }

    Scene_Puzzle.prototype.createChoseImageCommandWindow = function() {
        this._choseImageCommandWindow = new Window_ChoseImageCommand(0, 0)
        puzzleImages.forEach(function(image) {
            this._choseImageCommandWindow.setHandler(image.name, this.onChoseImage.bind(this, image.name))
        }, this)
        this._choseImageCommandWindow.setHandler('cancel', this.onPuzzleAfterTerminate.bind(this))
        this._choseImageCommandWindow.activate()
        this.addWindow(this._choseImageCommandWindow)
    }

    Scene_Puzzle.prototype.createChoseSizeCommandWindow = function() {
        this._choseSizeCommandWindow = new Window_ChoseSizeCommand(0, 0)
        this._choseSizeCommandWindow.setHandler('4', this.onChoseSize.bind(this, imageWidth / 4))
        this._choseSizeCommandWindow.setHandler('8', this.onChoseSize.bind(this, imageWidth / 8))
        this._choseSizeCommandWindow.setHandler('16', this.onChoseSize.bind(this, imageWidth / 16))
        this._choseSizeCommandWindow.setHandler('cancel', this.onSizeCancel.bind(this))
        this.addWindow(this._choseSizeCommandWindow)
    }

    Scene_Puzzle.prototype.createPuzzleWindow = function() {
        this._puzzleWindow = new Window_Puzzle(0, 0, Graphics.boxWidth, Graphics.boxHeight)
        this._puzzleWindow.setHandler('cancel', this.onPuzzleCancel.bind(this))
        this._puzzleWindow.onBeforeTerminate(this.onPuzzleBeforeTerminate.bind(this))
        this._puzzleWindow.onAfterTerminate(this.onPuzzleAfterTerminate.bind(this))
        this.addWindow(this._puzzleWindow)
    }

    Scene_Puzzle.prototype.createCommandWindow = function() {
        this._commandWindow = new Window_PuzzleCommand(0, 0)
        this._commandWindow.setHandler('retry', this.onRetryOk.bind(this))
        this._commandWindow.setHandler('quit', this.onPuzzleAfterTerminate.bind(this))
        this._commandWindow.setHandler('cancel', this.onRetryCancel.bind(this))
        this.addWindow(this._commandWindow)
    }

    Scene_Puzzle.prototype.onRetryOk = function() {
        this._commandWindow.close()
        this._choseImageCommandWindow.open()
        this._choseImageCommandWindow.activate()
        this._puzzleWindow.close()
        this._puzzleWindow.deactivate()
        this._puzzleWindow.clear()
    }

    Scene_Puzzle.prototype.onRetryCancel = function() {
        this._commandWindow.close()
        this._commandWindow.deactivate()
        this._puzzleWindow.activate()
    }

    Scene_Puzzle.prototype.onChoseImage = function(image) {
        this._puzzleWindow.setImage(image)
        this._choseSizeCommandWindow.open()
        this._choseSizeCommandWindow.activate()
        this._choseImageCommandWindow.deactivate()
    }

    Scene_Puzzle.prototype.onChoseSize = function(size) {
        this._puzzleWindow.open()
        this._puzzleWindow.activate()
        this._puzzleWindow.setSize(size)
        this._puzzleWindow.startPuzzle()
        this._choseSizeCommandWindow.close()
        this._choseSizeCommandWindow.deactivate()
    }

    Scene_Puzzle.prototype.onSizeCancel = function() {
        this._choseSizeCommandWindow.close()
        this._choseSizeCommandWindow.deactivate()
        this._choseImageCommandWindow.open()
        this._choseImageCommandWindow.activate()
    }

    Scene_Puzzle.prototype.onPuzzleCancel = function() {
        this._commandWindow.open()
        this._commandWindow.activate()
        this._commandWindow.select(0)
        this._puzzleWindow.deactivate()
    }

    Scene_Puzzle.prototype.onPuzzleBeforeTerminate = function() {
        delete this._puzzleWindow._handlers['cancel']
    }

    Scene_Puzzle.prototype.onPuzzleAfterTerminate = function() {
        this.popScene()
    }

    //=============================================================================
    // Window_ChoseImageCommand
    //=============================================================================
    function Window_ChoseImageCommand() {
        this.initialize.apply(this, arguments)
    }

    Window_ChoseImageCommand.prototype = Object.create(Window_Command.prototype)
    Window_ChoseImageCommand.prototype.constructor = Window_PuzzleCommand

    Window_ChoseImageCommand.prototype.windowWidth = function() {
        return Graphics.boxWidth
    }

    Window_ChoseImageCommand.prototype.windowHeight = function() {
        return Graphics.boxHeight
    }

    Window_ChoseImageCommand.prototype.makeCommandList = function() {
        puzzleImages.forEach(function(image) {
            this.addCommand('', image.name, true, { bitmap: image.bitmap })
        }, this)
    }

    Window_ChoseImageCommand.prototype._updateCursor = function() {}
    
    Window_ChoseImageCommand.prototype._refreshCursor = function() {}

    Window_ChoseImageCommand.prototype.maxCols = function() {
        return 4
    }

    Window_ChoseImageCommand.prototype.lineHeight = function() {
        return (this.height - this.padding * 2) / 4
    }

    Window_ChoseImageCommand.prototype.drawItem = function(index) {
        var rect = this.itemRect(index)
        var bitmap = this._list[index].ext.bitmap
        var ratio = bitmap.width / bitmap.height
        var margin = 8
        var itemHeight = rect.height - 2 * margin
        var itemWidth = itemHeight * ratio
        var x = rect.x + Math.round((rect.width - itemWidth) / 2)
        var y = rect.y + margin
        this.contents.blt(bitmap, 0, 0, bitmap.width, bitmap.height, x, y, itemWidth, itemHeight)
    }

    //=============================================================================
    // Window_ChoseSizeCommand
    //=============================================================================
    function Window_ChoseSizeCommand() {
        this.initialize.apply(this, arguments)
    }

    Window_ChoseSizeCommand.prototype = Object.create(Window_Command.prototype)
    Window_ChoseSizeCommand.prototype.constructor = Window_PuzzleCommand

    Window_ChoseSizeCommand.prototype.initialize = function(x, y) {
        Window_Command.prototype.initialize.call(this, x, y)
        this.openness = 0
        this.updatePlacement()
    }

    Window_ChoseSizeCommand.prototype.updatePlacement = function() {
        this.x = (Graphics.boxWidth - this.windowWidth()) / 2
        this.y = (Graphics.boxHeight - this.windowHeight()) / 2
    }

    Window_ChoseSizeCommand.prototype.makeCommandList = function() {
        this.addCommand(`${(imageWidth * imageHeight) / Math.pow(imageWidth / 4, 2)} pieces`, '4')
        this.addCommand(`${(imageWidth * imageHeight) / Math.pow(imageWidth / 8, 2)} pieces`, '8')
        this.addCommand(`${(imageWidth * imageHeight) / Math.pow(imageWidth / 16, 2)} pieces`, '16')
    }

    //=============================================================================
    // Window_Puzzle
    //=============================================================================
    function Window_Puzzle() {
        this.initialize.apply(this, arguments)
    }

    Window_Puzzle.DELAY_BEFORE_START = 60
    Window_Puzzle.CUTTING_PIECES_TIMER = 60
    Window_Puzzle.ROTATIONS = [0, Math.PI / 2, Math.PI, (3 * Math.PI) / 2]
    Window_Puzzle.BORDER_TYPES = ['inner', 'outer']

    Window_Puzzle.prototype = Object.create(Window_Selectable.prototype)
    Window_Puzzle.prototype.constructor = Window_Puzzle

    Window_Puzzle.prototype.initialize = function(x, y, width, height) {
        this._puzzleX = (Graphics.boxWidth - imageWidth) / 2
        this._puzzleY = (Graphics.boxHeight - imageHeight) / 2
        Window_Selectable.prototype.initialize.call(this, x, y, width, height)
        this.openness = 0
        this._pieces = []
        this.clear()
    }

    Window_Puzzle.prototype._refreshFrame = function() {
        Window.prototype._refreshFrame.call(this)
        var borderSize = 2
        this._windowFrameSprite.bitmap.fillRect(
            this._puzzleX - borderSize,
            this._puzzleY - borderSize,
            imageWidth + 2 * borderSize,
            imageHeight + 2 * borderSize,
            'black'
        )
        this._windowFrameSprite.bitmap.fillRect(this._puzzleX, this._puzzleY, imageWidth, imageHeight, '#111111')
    }

    Window_Puzzle.prototype.onBeforeTerminate = function(callback) {
        this._beforeTerminateCallback = callback
    }

    Window_Puzzle.prototype.onAfterTerminate = function(callback) {
        this._afterTerminateCallback = callback
    }

    Window_Puzzle.prototype.standardBackOpacity = function() {
        return 255
    }

    Window_Puzzle.prototype.clear = function() {
        this._image = null
        this._delayBeforeStart = null
        this._cuttingPiecesTimer = null

        this._isStarting = false
        this._isStarted = false
        this._isTerminated = false
        this._canTerminate = false

        this._currentPiece = null
        this._previousX = null
        this._previousY = null
        this._hasMoved = false
        this._pieces = []
        this.removeChild(this._layingPiecesSprite)
        this.removeChild(this._floatingPiecesSprite)
        this.removeChild(this._imageSprite)
        this.contents.clear()
    }

    Window_Puzzle.prototype.standardPadding = function() {
        return 0
    }

    Window_Puzzle.prototype.setImage = function(image) {
        this._image = ImageManager.loadBitmap(imgDirectory + '/', image)
    }

    Window_Puzzle.prototype.setSize = function(size) {
        this._size = size

        this._minX = -this._puzzleX + this._size / 2
        this._maxX = Graphics.boxWidth - this._puzzleX - this._size / 2

        this._minY = -this._puzzleY + this._size / 2
        this._maxY = Graphics.boxHeight - this._puzzleY - this._size / 2
    }

    Window_Puzzle.prototype.startPuzzle = function() {
        this.addImageSprite()
        this.addLayingPiecesSprite()
        this.addFloatingPiecesSprite()

        this.cutPieces()
        this._delayBeforeStart = Window_Puzzle.DELAY_BEFORE_START
    }

    Window_Puzzle.prototype.addImageSprite = function() {
        var bitmap = new Bitmap(imageWidth, imageHeight)
        bitmap.blt(this._image, 0, 0, this._image.width, this._image.height, 0, 0, imageWidth, imageHeight)
        this._imageSprite = new Sprite(bitmap)
        this._imageSprite.opacity = 0
        this._imageSprite.move(this._puzzleX, this._puzzleY)
        this.addChild(this._imageSprite)
    }

    Window_Puzzle.prototype.addLayingPiecesSprite = function() {
        this._layingPiecesSprite = new Sprite()
        this._layingPiecesSprite.opacity = 127
        this._layingPiecesSprite.move(this._puzzleX, this._puzzleY)
        this.addChild(this._layingPiecesSprite)
    }

    Window_Puzzle.prototype.addFloatingPiecesSprite = function() {
        this._floatingPiecesSprite = new Sprite()
        this._floatingPiecesSprite.move(this._puzzleX, this._puzzleY)
        this.addChild(this._floatingPiecesSprite)
    }

    Window_Puzzle.prototype.terminatePuzzle = function() {
        AudioManager.playMe({ name: victoryME, pan: 0, pitch: 100, volume: 100 })
        this.setVictoryVariableValue()
        this._imageSprite.opacity = 255
        this.removeChild(this._layingPiecesSprite)
        this.removeChild(this._floatingPiecesSprite)
        if (typeof this._beforeTerminateCallback === 'function') {
            this._beforeTerminateCallback.call(this)
        }
    }

    Window_Puzzle.prototype.setVictoryVariableValue = function() {
        var tolerance = $gameVariables.value(toleranceVariableId)
        var value = this._nbPieces + tolerance.padZero(3)
        $gameVariables.setValue(victoryVariableId, +value)
    }

    Window_Puzzle.prototype.cutPieces = function() {
        this.contents.clear()
        var width = imageWidth / this._size
        var height = imageHeight / this._size
        var piecesByPosition = {}
        for (var i = 0; i < width; i++) {
            piecesByPosition[i] = piecesByPosition[i] || {}
            for (var j = 0; j < height; j++) {
                var sprite = new Sprite(new Bitmap(2 * this._size, 2 * this._size))
                sprite.anchor.x = 0.5
                sprite.anchor.y = 0.5
                var piece = {
                    source: {
                        x: i * this._size - this._size / 2 + sprite.width / 2,
                        y: j * this._size - this._size / 2 + sprite.height / 2,
                        rotation: sprite.rotation
                    },
                    destination: {
                        x: Math.random() * (imageWidth - this._size * 2) + sprite.width / 2,
                        y: Math.random() * (imageHeight - this._size * 2) + sprite.height / 2,
                        rotation: Window_Puzzle.ROTATIONS[Math.floor(Math.random() * Window_Puzzle.ROTATIONS.length)]
                    },
                    x: null,
                    y: null,
                    width: this._size * 2,
                    height: this._size * 2,
                    sprite: sprite,
                    borders: {
                        left: this.leftBorder(piecesByPosition, i, j),
                        top: this.topBorder(piecesByPosition, i, j),
                        right: this.rightBorder(width, i),
                        bottom: this.bottomBorder(height, j)
                    }
                }
                piece.sprite.move(piece.source.x, piece.source.y)
                this.cutPiece(piece, i, j, width, height)
                this._floatingPiecesSprite.addChild(piece.sprite)
                this._pieces.push(piece)
                piecesByPosition[i][j] = piece
            }
        }

        this._nbPieces = this._pieces.length
    }

    Window_Puzzle.prototype.leftBorder = function(piecesByPosition, i, j) {
        return i > 0 ? piecesByPosition[i - 1][j].borders.right : 'flat'
    }

    Window_Puzzle.prototype.topBorder = function(piecesByPosition, i, j) {
        return j > 0 ? piecesByPosition[i][j - 1].borders.bottom : 'flat'
    }

    Window_Puzzle.prototype.rightBorder = function(width, i) {
        return i < width - 1 ? Window_Puzzle.BORDER_TYPES[Math.floor(Math.random() * Window_Puzzle.BORDER_TYPES.length)] : 'flat'
    }

    Window_Puzzle.prototype.bottomBorder = function(height, j) {
        return j < height - 1 ? Window_Puzzle.BORDER_TYPES[Math.floor(Math.random() * Window_Puzzle.BORDER_TYPES.length)] : 'flat'
    }

    Window_Puzzle.prototype.cutPiece = function(piece, i, j, width, height) {
        piece.sprite.bitmap._context.save()
        piece.sprite.bitmap._context.beginPath()
        let px = this._size / 2
        let py = this._size / 2
        let ps = this._size
        piece.sprite.bitmap._context.moveTo(px, py)
        this.clipBorder(piece, 'top', px, py, ps)
        this.clipBorder(piece, 'right', px, py, ps)
        this.clipBorder(piece, 'bottom', px, py, ps)
        this.clipBorder(piece, 'left', px, py, ps)
        piece.sprite.bitmap._context.clip()
        var xFactor = imageWidth / this._image.width
        var yFactor = imageHeight / this._image.height
        var dw = i === 0 || i === width - 1 ? (3 * piece.sprite.bitmap.width) / 4 : piece.sprite.bitmap.width
        var dh = j === 0 || j === height - 1 ? (3 * piece.sprite.bitmap.height) / 4 : piece.sprite.bitmap.height
        var dx = i === 0 ? this._size / 2 : 0
        var dy = j === 0 ? this._size / 2 : 0
        var sw = dw / xFactor
        var sh = dh / yFactor
        var sx = (i === 0 ? 0 : i * this._size - this._size / 2) / xFactor
        var sy = (j === 0 ? 0 : j * this._size - this._size / 2) / yFactor
        piece.sprite.bitmap.blt(this._image, sx, sy, sw, sh, dx, dy, dw, dh)
        piece.sprite.bitmap._context.lineWidth = 2
        piece.sprite.bitmap._context.lineJoin = 'round'
        piece.sprite.bitmap._context.lineCap = 'round'
        piece.sprite.bitmap._context.shadowBlur = 2
        piece.sprite.bitmap._context.shadowColor = 'rgb(0, 0, 0)'
        piece.sprite.bitmap._context.stroke()
    }

    // https://stackoverflow.com/questions/30617132/jigsaw-puzzle-pices-using-bezier-curve
    // Given the center point of the piece (cx,cy) and the side length (s)
    // The single side "outy" design is below
    // Use this single design (with transforms/mirroring) to make all pieces
    // ctx.lineTo(cx + s * .34, cy);
    // ctx.bezierCurveTo(cx + s * .5, cy, cx + s * .4, cy + s * -.15, cx + s * .4, cy + s * -.15);
    // ctx.bezierCurveTo(cx + s * .3, cy + s * -.3, cx + s * .5, cy + s * -.3, cx + s * .5, cy + s * -.3);
    // ctx.bezierCurveTo(cx + s * .7, cy + s * -.3, cx + s * .6, cy + s * -.15, cx + s * .6, cy + s * -.15);
    // ctx.bezierCurveTo(cx + s * .5, cy, cx + s * .65, cy, cx + s * .65, cy);
    // ctx.lineTo(cx + s, cy);
    Window_Puzzle.prototype.clipBorder = function(piece, side, x, y, size) {
        let size3 = size * 0.3
        let size4 = size * 0.4
        let size5 = size * 0.5
        let size6 = size * 0.6
        let size7 = size * 0.7
        let size15 = size * 0.15
        let size34 = size * 0.34
        let size65 = size * 0.65
        if (side === 'top') {
            let xt = x
            let yt = y
            if (piece.borders[side] === 'outer') {
                piece.sprite.bitmap._context.lineTo(xt + size34, yt)
                piece.sprite.bitmap._context.bezierCurveTo(xt + size5, yt, xt + size4, yt + size15, xt + size4, yt + size15)
                piece.sprite.bitmap._context.bezierCurveTo(xt + size3, yt + size3, xt + size5, yt + size3, xt + size5, yt + size3)
                piece.sprite.bitmap._context.bezierCurveTo(xt + size7, yt + size3, xt + size6, yt + size15, xt + size6, yt + size15)
                piece.sprite.bitmap._context.bezierCurveTo(xt + size5, yt, xt + size65, yt, xt + size65, yt)
            } else if (piece.borders[side] === 'inner') {
                piece.sprite.bitmap._context.lineTo(xt + size34, yt)
                piece.sprite.bitmap._context.bezierCurveTo(xt + size5, yt, xt + size4, yt - size15, xt + size4, yt - size15)
                piece.sprite.bitmap._context.bezierCurveTo(xt + size3, yt - size3, xt + size5, yt - size3, xt + size5, yt - size3)
                piece.sprite.bitmap._context.bezierCurveTo(xt + size7, yt - size3, xt + size6, yt - size15, xt + size6, yt - size15)
                piece.sprite.bitmap._context.bezierCurveTo(xt + size5, yt, xt + size65, yt, xt + size65, yt)
            }
            piece.sprite.bitmap._context.lineTo(x + size, yt)
            return
        }
        if (side === 'right') {
            let xr = x + size
            let yr = y
            if (piece.borders[side] === 'outer') {
                piece.sprite.bitmap._context.lineTo(xr, yr + size34)
                piece.sprite.bitmap._context.bezierCurveTo(xr, yr + size5, xr + size15, yr + size4, xr + size15, yr + size4)
                piece.sprite.bitmap._context.bezierCurveTo(xr + size3, yr + size3, xr + size3, yr + size5, xr + size3, yr + size5)
                piece.sprite.bitmap._context.bezierCurveTo(xr + size3, yr + size7, xr + size15, yr + size6, xr + size15, yr + size6)
                piece.sprite.bitmap._context.bezierCurveTo(xr, yr + size5, xr, yr + size65, xr, yr + size65)
            } else if (piece.borders[side] === 'inner') {
                piece.sprite.bitmap._context.lineTo(xr, yr + size34)
                piece.sprite.bitmap._context.bezierCurveTo(xr, yr + size5, xr - size15, yr + size4, xr - size15, yr + size4)
                piece.sprite.bitmap._context.bezierCurveTo(xr - size3, yr + size3, xr - size3, yr + size5, xr - size3, yr + size5)
                piece.sprite.bitmap._context.bezierCurveTo(xr - size3, yr + size7, xr - size15, yr + size6, xr - size15, yr + size6)
                piece.sprite.bitmap._context.bezierCurveTo(xr, yr + size5, xr, yr + size65, xr, yr + size65)
            }
            piece.sprite.bitmap._context.lineTo(xr, yr + size)
            return
        }
        if (side === 'bottom') {
            let xb = x + size
            let yb = y + size
            if (piece.borders[side] === 'outer') {
                piece.sprite.bitmap._context.lineTo(xb - size34, yb)
                piece.sprite.bitmap._context.bezierCurveTo(xb - size5, yb, xb - size4, yb + size15, xb - size4, yb + size15)
                piece.sprite.bitmap._context.bezierCurveTo(xb - size3, yb + size3, xb - size5, yb + size3, xb - size5, yb + size3)
                piece.sprite.bitmap._context.bezierCurveTo(xb - size7, yb + size3, xb - size6, yb + size15, xb - size6, yb + size15)
                piece.sprite.bitmap._context.bezierCurveTo(xb - size5, yb, xb - size65, yb, xb - size65, yb)
            } else if (piece.borders[side] === 'inner') {
                piece.sprite.bitmap._context.lineTo(xb - size34, yb)
                piece.sprite.bitmap._context.bezierCurveTo(xb - size5, yb, xb - size4, yb - size15, xb - size4, yb - size15)
                piece.sprite.bitmap._context.bezierCurveTo(xb - size3, yb - size3, xb - size5, yb - size3, xb - size5, yb - size3)
                piece.sprite.bitmap._context.bezierCurveTo(xb - size7, yb - size3, xb - size6, yb - size15, xb - size6, yb - size15)
                piece.sprite.bitmap._context.bezierCurveTo(xb - size5, yb, xb - size65, yb, xb - size65, yb)
            }
            piece.sprite.bitmap._context.lineTo(x, yb)
            return
        }
        if (side === 'left') {
            let xl = x
            let yl = y + size
            if (piece.borders[side] === 'outer') {
                piece.sprite.bitmap._context.lineTo(xl, yl - size34)
                piece.sprite.bitmap._context.bezierCurveTo(xl, yl - size5, xl + size15, yl - size4, xl + size15, yl - size4)
                piece.sprite.bitmap._context.bezierCurveTo(xl + size3, yl - size3, xl + size3, yl - size5, xl + size3, yl - size5)
                piece.sprite.bitmap._context.bezierCurveTo(xl + size3, yl - size7, xl + size15, yl - size6, xl + size15, yl - size6)
                piece.sprite.bitmap._context.bezierCurveTo(xl, yl - size5, xl, yl - size65, xl, yl - size65)
            } else if (piece.borders[side] === 'inner') {
                piece.sprite.bitmap._context.lineTo(xl, yl - size34)
                piece.sprite.bitmap._context.bezierCurveTo(xl, yl - size5, xl - size15, yl - size4, xl - size15, yl - size4)
                piece.sprite.bitmap._context.bezierCurveTo(xl - size3, yl - size3, xl - size3, yl - size5, xl - size3, yl - size5)
                piece.sprite.bitmap._context.bezierCurveTo(xl - size3, yl - size7, xl - size15, yl - size6, xl - size15, yl - size6)
                piece.sprite.bitmap._context.bezierCurveTo(xl, yl - size5, xl, yl - size65, xl, yl - size65)
            }
            piece.sprite.bitmap._context.lineTo(xl, y)
        }
    }

    Window_Puzzle.prototype.update = function() {
        Window_Selectable.prototype.update.call(this)
        if (!this._image) {
            return
        }

        if (this._delayBeforeStart > 0) {
            this._delayBeforeStart--
        } else if (this._delayBeforeStart != null) {
            this._delayBeforeStart = null
            this._isStarting = true
        }

        if (this._isStarting) {
            this._isStarting = false
            this._cuttingPiecesTimer = Window_Puzzle.CUTTING_PIECES_TIMER
        }

        if (this._cuttingPiecesTimer > 0) {
            this._cuttingPiecesTimer--
            var easeInOut = this.easeInOutCubic(1 - this._cuttingPiecesTimer / Window_Puzzle.CUTTING_PIECES_TIMER)
            this._pieces.forEach(function(piece) {
                this.updatePiece(piece, easeInOut)
            }, this)
        } else if (this._cuttingPiecesTimer != null) {
            this._cuttingPiecesTimer = null
            this._isStarted = true
        }

        if (this._isStarted) {
            if (!this._currentPiece) {
                if (TouchInput.isPressed()) {
                    this._pressedTime = 0
                    this._previousX = TouchInput.x
                    this._previousY = TouchInput.y
                    for (let i = this._pieces.length - 1; i >= 0; i--) {
                        let onPiece = this.onPiece(this._pieces[i], TouchInput.x, TouchInput.y)
                        if (onPiece) {
                            this._currentPiece = this._pieces[i]
                            this._floatingPiecesSprite.removeChild(this._currentPiece.sprite)
                            this._floatingPiecesSprite.addChild(this._currentPiece.sprite)
                            this._pieces.splice(i, 1)
                            this._pieces.push(this._currentPiece)
                            break
                        }
                    }
                }
            } else {
                if (TouchInput.isReleased()) {
                    if (!this._hasMoved) {
                        this.updatePieceRotation(this._currentPiece, this._currentPiece.sprite.rotation + Math.PI / 2)
                    }
                    this.checkPosition(this._currentPiece)
                    this._currentPiece = null
                    this._hasMoved = false
                } else if (this._previousX !== TouchInput.x || this._previousY !== TouchInput.y) {
                    this._hasMoved = true
                    this.updatePiecePosition(
                        this._currentPiece,
                        this._currentPiece.sprite.x + TouchInput.x - this._previousX,
                        this._currentPiece.sprite.y + TouchInput.y - this._previousY
                    )
                    this._previousX = TouchInput.x
                    this._previousY = TouchInput.y
                }
            }

            if (!this._pieces.length) {
                this._isStarted = false
                this.terminatePuzzle()
                this._isTerminated = true
            }
        }

        if (this._isTerminated && !AudioManager._meBuffer) {
            this._isTerminated = false
            this._canTerminate = true
        }

        if (this._canTerminate && TouchInput.isPressed() && typeof this._afterTerminateCallback === 'function') {
            this._afterTerminateCallback.call(this)
        }
    }

    Window_Puzzle.prototype.onPiece = function(piece, x, y) {
        return (
            x - this._puzzleX > piece.sprite.x - piece.sprite.width / 2 + this._size / 2 &&
            x - this._puzzleX < piece.sprite.x + piece.sprite.width / 2 - this._size / 2 &&
            y - this._puzzleY > piece.sprite.y - piece.sprite.height / 2 + this._size / 2 &&
            y - this._puzzleY < piece.sprite.y + piece.sprite.height / 2 - this._size / 2
        )
    }

    Window_Puzzle.prototype.checkPosition = function(piece) {
        let tolerance = $gameVariables.value(toleranceVariableId)
        var t = (tolerance / 100) * this._size
        if (
            Math.abs(piece.source.x - piece.sprite.x) < t &&
            Math.abs(piece.source.y - piece.sprite.y) < t &&
            piece.source.rotation === piece.sprite.rotation
        ) {
            this._floatingPiecesSprite.removeChild(piece.sprite)
            this._layingPiecesSprite.addChild(piece.sprite)
            let index = this._pieces.indexOf(piece)
            if (index > -1) {
                this._pieces.splice(index, 1)
            }
            this.updatePiecePosition(piece, piece.source.x, piece.source.y)
        }
    }

    Window_Puzzle.prototype.updatePiece = function(piece, easeInOut) {
        var x = piece.source.x + (piece.destination.x - piece.source.x) * easeInOut
        var y = piece.source.y + (piece.destination.y - piece.source.y) * easeInOut
        var rotation = piece.source.rotation + (piece.destination.rotation - piece.source.rotation) * easeInOut
        this.updatePiecePosition(piece, x, y)
        this.updatePieceRotation(piece, rotation)
    }

    Window_Puzzle.prototype.updatePiecePosition = function(piece, x, y) {
        x = Math.max(this._minX, Math.min(this._maxX, x))
        y = Math.max(this._minY, Math.min(this._maxY, y))
        piece.sprite.move(x, y)
    }

    Window_Puzzle.prototype.updatePieceRotation = function(piece, rotation) {
        piece.sprite.rotation = rotation % (Math.PI * 2)
    }

    Window_Puzzle.prototype.easeInOutCubic = function(t) {
        return t < 0.5 ? 4 * t * t * t : (t - 1) * (2 * t - 2) * (2 * t - 2) + 1
    }

    //=============================================================================
    // Window_PuzzleCommand
    //=============================================================================
    function Window_PuzzleCommand() {
        this.initialize.apply(this, arguments)
    }

    Window_PuzzleCommand.prototype = Object.create(Window_Command.prototype)
    Window_PuzzleCommand.prototype.constructor = Window_PuzzleCommand

    Window_PuzzleCommand.prototype.initialize = function(x, y) {
        Window_Command.prototype.initialize.call(this, x, y)
        this.openness = 0
        this.updatePlacement()
    }

    Window_PuzzleCommand.prototype.updatePlacement = function() {
        this.x = (Graphics.boxWidth - this.windowWidth()) / 2
        this.y = (Graphics.boxHeight - this.windowHeight()) / 2
    }

    Window_PuzzleCommand.prototype.makeCommandList = function() {
        this.addCommand(retryText, 'retry')
        this.addCommand(quitText, 'quit')
    }

    //=============================================================================
    // Scene_Boot
    //=============================================================================
    var Scene_Boot_create = Scene_Boot.prototype.create
    Scene_Boot.prototype.create = function() {
        Scene_Boot_create.call(this)
        puzzleImages.forEach(function(image) {
            image.bitmap = ImageManager.reserveBitmap(imgDirectory + '/', image.name)
        }, this)
    }

    //=============================================================================
    // Game_Interpreter
    //=============================================================================
    var Game_Interpreter_pluginCommand = Game_Interpreter.prototype.pluginCommand
    Game_Interpreter.prototype.pluginCommand = function(command, args) {
        Game_Interpreter_pluginCommand.call(this, command, args)
        if (command === 'Puzzle' && args[0] === 'open') {
            $gameVariables.setValue(victoryVariableId, 0)
            SceneManager.push(Scene_Puzzle)
        }
    }
})()
